@@ -213,6 +213,9 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
                     com.openexchange.file.storage.boxcom.BoxFile boxFile = new com.openexchange.file.storage.boxcom.BoxFile(folderId, id, userId, rootFolderId).parseBoxFile(fileInfo);
                     return boxFile;
                 } catch (final BoxAPIException e) {
+                    if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                        throw e;
+                    }
                     throw handleHttpResponseError(id, account.getId(), e);
                 }
             }
@@ -237,12 +240,51 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
                         Info fileInfo = boxFile.getInfo();
                         checkFileValidity(fileInfo);
 
-                        fileInfo.setName(file.getFileName());
-                        fileInfo.setDescription(file.getDescription());
-                        boxFile.updateInfo(fileInfo);
+                        String name = null;
+                        if (null == modifiedFields || modifiedFields.contains(Field.FILENAME)) {
+                            name = file.getFileName();
+                            if (Strings.isEmpty(name)) {
+                                name = null;
+                            }
+                        }
+
+                        String description = null;
+                        if (null == modifiedFields || modifiedFields.contains(Field.DESCRIPTION)) {
+                            description = file.getDescription();
+                            if (Strings.isEmpty(description)) {
+                                description = null;
+                            }
+                        }
+
+                        if (null != name || null != description) {
+                            // Optimistic rename...
+                            FileName fn = null;
+                            boolean retry;
+                            do {
+                                try {
+                                    retry = false;
+                                    fileInfo.setName(name);
+                                    fileInfo.setDescription(description);
+                                    boxFile.updateInfo(fileInfo);
+                                } catch (BoxAPIException e) {
+                                    if (409 != e.getResponseCode() || null == name) {
+                                        throw e;
+                                    }
+                                    // API returned conflict. Compose another name and retry
+                                    if (null == fn) {
+                                        fn = new FileName(name);
+                                    }
+                                    name = fn.enhance();
+                                    retry = true;
+                                }
+                            } while (retry);
+                        }
 
                         return new IDTuple(file.getFolderId(), boxFile.getID());
                     } catch (final BoxAPIException e) {
+                        if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                            throw e;
+                        }
                         throw handleHttpResponseError(file.getId(), account.getId(), e);
                     }
                 }
@@ -271,42 +313,30 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
                     String boxFolderId = toBoxFolderId(destFolder);
                     com.box.sdk.BoxFolder boxFolder = new com.box.sdk.BoxFolder(boxClient, boxFolderId);
 
-                    // Check destination folder
-                    String title = fileInfo.getName();
-                    {
-                        String baseName;
-                        String ext;
-                        {
-                            int dotPos = title.lastIndexOf('.');
-                            if (dotPos > 0) {
-                                baseName = title.substring(0, dotPos);
-                                ext = title.substring(dotPos);
-                            } else {
-                                baseName = title;
-                                ext = "";
+                    String name = fileInfo.getName();
+                    FileName fn = null;
+                    Info copiedFile = null;
+                    do {
+                        try {
+                            // Copy the file
+                            copiedFile = boxFile.copy(boxFolder, name);
+                        } catch (BoxAPIException e) {
+                            if (409 != e.getResponseCode() || null == name) {
+                                throw e;
                             }
-                        }
-
-                        int count = 1;
-                        boolean keepOn = true;
-                        while (keepOn) {
-                            keepOn = false;
-                            for (com.box.sdk.BoxItem.Info info : boxFolder) {
-                                // Check for filename clashes and append a number at the end of the filename
-                                if (info instanceof Info && title.equals(info.getName())) {
-                                    keepOn = true;
-                                    title = new StringBuilder(baseName).append(" (").append(count++).append(')').append(ext).toString();
-                                    break;
-                                }
+                            // API returned conflict. Compose another name and retry
+                            if (null == fn) {
+                                fn = new FileName(name);
                             }
+                            name = fn.enhance();
                         }
-                    }
-
-                    // Copy the file
-                    Info copiedFile = boxFile.copy(boxFolder, title);
+                    } while (null == copiedFile);
 
                     return new IDTuple(destFolder, copiedFile.getID());
                 } catch (final BoxAPIException e) {
+                    if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                        throw e;
+                    }
                     throw handleHttpResponseError(source.getId(), account.getId(), e);
                 }
             }
@@ -328,42 +358,30 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
                     String boxFolderId = toBoxFolderId(destFolder);
                     com.box.sdk.BoxFolder boxFolder = new com.box.sdk.BoxFolder(apiConnection, boxFolderId);
 
-                    // Check destination folder
-                    String title = fileInfo.getName();
-                    {
-                        String baseName;
-                        String ext;
-                        {
-                            int dotPos = title.lastIndexOf('.');
-                            if (dotPos > 0) {
-                                baseName = title.substring(0, dotPos);
-                                ext = title.substring(dotPos);
-                            } else {
-                                baseName = title;
-                                ext = "";
+                    String name = fileInfo.getName();
+                    FileName fn = null;
+                    com.box.sdk.BoxItem.Info movedFile = null;
+                    do {
+                        try {
+                            // Copy the file
+                            movedFile = boxFile.move(boxFolder, name);
+                        } catch (BoxAPIException e) {
+                            if (409 != e.getResponseCode() || null == name) {
+                                throw e;
                             }
-                        }
-                        int count = 1;
-                        boolean keepOn = true;
-                        while (keepOn) {
-                            keepOn = false;
-                            // TODO: Expensive as it needs to iterate through all the files in a folder...
-                            //       Consider the possibility of checking if the file exists in the folder with the exists method
-                            for (com.box.sdk.BoxItem.Info info : boxFolder) {
-                                // Check for filename clashes and append a number at the end of the filename
-                                if (info instanceof Info && title.equals(info.getName())) {
-                                    keepOn = true;
-                                    title = new StringBuilder(baseName).append(" (").append(count++).append(')').append(ext).toString();
-                                    break;
-                                }
+                            // API returned conflict. Compose another name and retry
+                            if (null == fn) {
+                                fn = new FileName(name);
                             }
+                            name = fn.enhance();
                         }
-                    }
-
-                    com.box.sdk.BoxItem.Info movedFile = boxFile.move(boxFolder, title);
+                    } while (null == movedFile);
 
                     return new IDTuple(destFolder, movedFile.getID());
                 } catch (final BoxAPIException e) {
+                    if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                        throw e;
+                    }
                     throw handleHttpResponseError(source.getId(), account.getId(), e);
                 }
             }
@@ -421,6 +439,9 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
                     } catch (IOException e) {
                         throw FileStorageExceptionCodes.IO_ERROR.create(e, e.getMessage());
                     } catch (BoxAPIException e) {
+                        if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                            throw e;
+                        }
                         throw handleHttpResponseError(id, account.getId(), e);
                     }
                 }
@@ -438,6 +459,9 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
                     error = true; // Avoid premature closing
                     return stream;
                 } catch (final BoxAPIException e) {
+                    if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                        throw e;
+                    }
                     throw handleHttpResponseError(id, account.getId(), e);
                 } finally {
                     if (error) {
@@ -469,6 +493,9 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
 
                     return new ByteArrayInputStream(thumbnail);
                 } catch (final BoxAPIException e) {
+                    if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                        throw e;
+                    }
                     throw handleHttpResponseError(id, account.getId(), e);
                 }
             }
@@ -578,6 +605,9 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
 
                     return new IDTuple(file.getFolderId(), fileInfo.getID());
                 } catch (BoxAPIException e) {
+                    if (SC_UNAUTHORIZED == e.getResponseCode()) {
+                        throw e;
+                    }
                     throw handleHttpResponseError(file.getId(), account.getId(), e);
                 }
             }
@@ -971,4 +1001,37 @@ public class BoxFileAccess extends AbstractBoxResourceAccess implements Thumbnai
             return parsedFields.toArray(new String[parsedFields.size()]);
         }
     }
+
+    private static class FileName {
+
+        private final String base;
+        private final String extension;
+        private int count;
+
+        FileName(String fileName) {
+            super();
+            int dot = fileName.lastIndexOf('.');
+            if (dot >= 0) {
+                base = fileName.substring(0, dot);
+                extension = fileName.substring(dot+1);
+            } else {
+                base = fileName;
+                extension = null;
+            }
+            count = 0;
+        }
+
+        String enhance() {
+            if (null == extension) {
+                return new StringBuilder(base).append('(').append(++count).append(')').toString();
+            }
+
+            return new StringBuilder(base).append('(').append(++count).append(')').append('.').append(extension).toString();
+        }
+
+        @Override
+        public String toString() {
+            return null == extension ? base : base + "." + extension;
+        }
+    }
 }
